@@ -3,7 +3,7 @@
  *
  * This file is part of fizmo.
  *
- * Copyright (c) 2009 Christoph Ender.
+ * Copyright (c) 2009-2011 Christoph Ender.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -91,7 +91,8 @@ static uint8_t first_word_found;
 static z_ucs z_ucs_output_buffer[Z_UCS_OUTPUT_BUFFER_SIZE];
 
 static uint8_t dictionary_entry_length;
-static uint16_t number_of_dictionary_entries;
+static int16_t number_of_dictionary_entries;
+static bool dictionary_is_unsorted;
 static uint8_t *dictionary_start;
 
 z_ucs z_ucs_newline_string[] = { Z_UCS_NEWLINE, 0 };
@@ -472,7 +473,7 @@ static uint8_t locate_dictionary_entry(
     }
   }
 
-  // FIXME: Improve searching
+  // FIXME: Improve searching (use dictionary_is_unsorted).
   for (i=0; i<number_of_dictionary_entries; i++)
   {
     /*
@@ -563,7 +564,6 @@ static void tokenise(
   uint8_t number_of_input_codes;
   uint8_t *input_codes;
 
-  uint8_t words_parsed = 0;
   uint8_t current_char;
   uint8_t space_found = 0;
   uint8_t non_space_seperator_found = 0;
@@ -587,7 +587,16 @@ static void tokenise(
   input_codes = dictionary;
   dictionary += number_of_input_codes;
   dictionary_entry_length = *(dictionary++);
-  number_of_dictionary_entries = load_word(dictionary);
+  number_of_dictionary_entries = (int16_t)load_word(dictionary);
+  if (number_of_dictionary_entries < 0)
+  {
+    dictionary_is_unsorted = true;
+    number_of_dictionary_entries = -number_of_dictionary_entries;
+  }
+  else
+  {
+    dictionary_is_unsorted = false;
+  }
   dictionary_start = dictionary + 2;
 
 #ifdef ENABLE_TRACING
@@ -606,7 +615,7 @@ static void tokenise(
 
   end_of_line_found = 0;
   //last_token_start_index = 0;
-  while ((words_parsed < maximum_words) && (end_of_line_found == 0))
+  while ((number_of_words_found < maximum_words) && (end_of_line_found == 0))
   {
     TRACE_LOG("Looking at %c.\n", z_text_buffer[z_text_buffer_offset]);
     TRACE_LOG("current-word-start: %d.\n", current_word_start);
@@ -672,7 +681,8 @@ static void tokenise(
       if (zchar_storage_symbols_stored > 0)
       {
         zchar_storage_finish();
-        number_of_words_found++;
+        if (++number_of_words_found == maximum_words)
+          break;
 
         (void)locate_dictionary_entry(tokenize_buffer,
             &parse_buffer_index,
@@ -686,7 +696,7 @@ static void tokenise(
 
       store_ZSCII_as_zchar(current_char);
       zchar_storage_finish();
-      number_of_words_found ++;
+      number_of_words_found++;
 
       (void)locate_dictionary_entry(
           tokenize_buffer,
@@ -713,7 +723,7 @@ static void tokenise(
             z_text_buffer_offset,
             last_token_start_index);
         */
-        number_of_words_found ++;
+        number_of_words_found++;
 
         (void)locate_dictionary_entry(
             tokenize_buffer,
